﻿
using System;
using System.Collections.Generic;
using System.Linq;
using Model.Cartography;
using Model.People;

namespace Model.Common
{
    public class Simulation
    {
        private static Simulation instance;
        public static Simulation Instance { get => instance; }

        public List<Person> People => people;


        private Config config;
        private readonly List<Person> people;
        private Map map;
        private TickResult currentTickResult;

        private Simulation(Config config) {
            instance = this;
            this.config = config;
            people = new List<Person>();
        }

        public static Simulation CreateOnce(Config config) {
            if (instance != null)
                throw new Exception("Simulation already created");
            else
                return new Simulation(config);
        }

        public int GetMoney() {
            return Park.Instance.Balance;
        }

        public IInspectable GetPark() {
            return Park.Instance;
        }

        public IBuilder GetBuilder() {
            return map;
        }

        public void AddPerson(Person person) {
            currentTickResult.newVisitors.Append((person, person));
            people.Append(person);
        }

        public TickResult Tick() {
            Park.Instance.Tick();
            people.ForEach(person => person.Tick());

            TickResult result = currentTickResult;
            result.gameOver = GetMoney() < 0;

            currentTickResult = new TickResult
            {
                newVisitors = new List<(IDisplayable, IInspectable)>(),
                newCleaningStaff = new List<(IDisplayable, IInspectable)>(),
                newMaintenanceStaff = new List<(IDisplayable, IInspectable)>()
            };

            return result;
        }


        [System.Serializable]
        public class Config
        {
            public float movementSpeed;
        }
    }

    
}
