using System.IO;
using System.Linq;
using NUnit.Framework;
using System.Runtime.CompilerServices;

using Model.Cartography.Nav;

namespace Tests.Model.Cartography.Nav {
    public class Test_NavigationGraph
    {
        private static readonly string TEST_DIR = Path.Combine("TestData", "Test_NavigationGraph");
        private static readonly string DATA_DIR = Path.Combine(TEST_DIR, "data");

#if DEBUG
        private StreamWriter DebugStream = new StreamWriter(Path.Combine(TEST_DIR, "output"));  
        /* hacks on hacks all the way */

        private void debug(string s, [CallerMemberName] string caller = null)
            => DebugStream?.WriteLine((caller != null ? caller + ": " : "") + s);

        public Test_NavigationGraph() {
            DebugStream.AutoFlush = true;
        }

        ~Test_NavigationGraph() {
            DebugStream.Close();
        }
#else
        private void debug(string s) { }
#endif

        [Test]
        public void TestAll()
        {
            debug($"> reading tests from {DATA_DIR}");

            foreach (var filename in Directory.GetFiles(DATA_DIR)) {
                NavigationGraph ng = new NavigationGraph();

#if DEBUG
                ng.DebugStream = DebugStream;
#endif
                debug($"> reading test {filename}");

                using (StreamReader file = new StreamReader(filename)) {
                    string line;
                    while ((line = file.ReadLine()) != null)
                        ProcessLine(ng, line);
                }

            }
        }

        private void ProcessLine(NavigationGraph ng, string line) {
            string[] parts = line.Split('=').Select(x => x.Trim()).ToArray();
            string[] cmd = parts[0].Split(' ');
            string[] res = parts[1].Split(' ');

            VertexHandle vh;

            debug($">> {line}");

            switch (cmd[0]) {
                case "add_vertex":
                    vh = ng.AddVertex();
                    Assert.AreEqual(int.Parse(res[0]), vh.ID);
                    break;
                case "add_edge":
                    ng.AddEdge(new VertexHandle(int.Parse(cmd[1])),
                               new VertexHandle(int.Parse(cmd[2])));
                    break;
                case "plan_route":
                    var rh = ng.PlanRoute(new VertexHandle(int.Parse(cmd[1])),
                                          new VertexHandle(int.Parse(cmd[2])));
                    Assert.AreEqual(int.Parse(res[0]), rh.ID);
                    break;
                case "get_next":
                    vh = ng.GetNextOnRoute(new RouteHandle(int.Parse(cmd[1])));
                    Assert.AreEqual(int.Parse(res[0]), vh.ID);
                    break;
                default:
                    Assert.Fail();
                    break;
            }
        }
    }
}
